<?php
namespace Teknisa\ACL\Controllers;

use Teknisa\Libs\Controller\CRUD;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Teknisa\Model\Util\NovoCodigo;
use Teknisa\ACL\Controllers\TeknisaCrud;

class OperadorPrincipalController extends TeknisaCrud {
    
    protected $dataSourceName = 'operadorPrincipal';
    protected $connection;
    protected $newCode;
    protected $operadorPrincipalService;
    
    protected $parameterBagColumns = array('P_IDOPERMASTER', 'OPERADOR_LOGADO', 'P_CDFILIAL', 'NRORG');
    
    public function __construct($crudService, $parameterBag, $operadorPrincipalService, $entityManager) {
        $this->connection               = $entityManager->getConnection();
        $this->operadorPrincipalService = $operadorPrincipalService;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function findCliente(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName      = 'cliente';
        $this->parameterBagColumns = array();
        parent::find($request, $response);
    }
    
    public function findVendedor(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName      = 'vendedor';
        $this->parameterBagColumns = array();
        parent::find($request, $response);
    }
    
    public function findEstruturam(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName      = 'estruturam';
        $this->parameterBagColumns = array();
        parent::find($request, $response);
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        foreach ($request->getDataSet()->getRows() as $row) {
            $this->connection->beginTransaction();
            try {
                
                if ($row['__is_new']) {
                    if ($row['CDOPERADOR'] == null) {
                        $row['CDOPERADOR'] = $this->getMaxOperCode($row, $response);
                    }
                    $row["CDSENHAOPERWEB"] = md5($row["CDSENHAOPERWEB"]);
                    
                    //Se não tiver sido selecionado parceiro, cria um novo seta o selecionado na row
                    if (empty($row['NRPARCNEGOCIO'])) {
                        $row['NRPARCNEGOCIO'] = NovoCodigo::geraCodigo($row["NRORG"], "PARCNEGOCIO", 1, 10, false, false);
                        $this->criaParcnegocio($row, $response);
                        
                        $row["NRPARCERIA"] = NovoCodigo::geraCodigo($row["NRORG"], "PARCERIA", 1, 10, false, false);
                        $this->criaParceria($row, $response, "OPERADOR");
                    }
                    
                    if ($row["CDCLIENTE"] != null) {
                        $row["NRPARCERIA"] = NovoCodigo::geraCodigo($row["NRORG"], "PARCERIA", 1, 10, false, false);
                        $this->criaParceria($row, $response, "INTERLOCUTOR");
                        
                        $row["NRRELACIONAPARC"] = NovoCodigo::geraCodigo($row["NRORG"], "RELACPARCNEG", 1, 10, false, false);
                        $this->criaRelacionaParc($row, $response);
                        
                        $row["CDINTECLIE"] = NovoCodigo::geraCodigo($row["NRORG"], 'INTECLIE' . $row["CDCLIENTE"], 1, 2, false, false);
                        $this->criaInteclie($row, $response);
                    }
                    
                    $row["NRORGANIZACAOOPER"] = NovoCodigo::geraCodigo($row["NRORG"], "ORGANIZACAOOPER", 1, 4, false, false);
                    $this->criaOrganOper($row, $response);
                    
                    parent::save($request, $response);
                    
                    if (isset($row["FILIAIS"])) {
                        $this->replicaFiliais($row, $response);
                    }
                    
                    $response->addDataSet(new DataSet("OPERADOR", array(
                        "CDOPERADOR" => $row["CDOPERADOR"],
                        "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
                        "CDSENHAOPERWEB" => $row["CDSENHAOPERWEB"]
                    )));
                } else {
                    if (($row["OLDNRINSCRICAOPARC"] != $row["NRINSCRICAOPARC"]) || ($row["OLDNMOPERADOR"] != $row["NMOPERADOR"])) {
                        $this->updateParcnegocio($row, $response);
                    }
                    
                    if ($row["CDCLIENTE"] != $row["OLDCDCLIENTE"]) {
                        $this->validarUpdateCliente($row, $response);
                    }
                    
                    if ($row["OLDCDSENHAOPERWEB"] != $row["CDSENHAOPERWEB"]) {
                        $row["CDSENHAOPERWEB"] = md5($row["CDSENHAOPERWEB"]);
                    }
                    
                    //Se não tiver sido selecionado parceiro, cria um novo seta o selecionado na row
                    if (empty($row['NRPARCNEGOCIO'])) {
                        $row['NRPARCNEGOCIO'] = NovoCodigo::geraCodigo($row["NRORG"], "PARCNEGOCIO", 1, 10, false, false);
                        $this->criaParcnegocio($row, $response);
                        
                        $row["NRPARCERIA"] = NovoCodigo::geraCodigo($row["NRORG"], "PARCERIA", 1, 10, false, false);
                        $this->criaParceria($row, $response, "OPERADOR");
                    }
                    
                    $response->addDataSet(new DataSet("OPERADOR", array(
                        "CDSENHAOPERWEB" => $row["CDSENHAOPERWEB"]
                    )));
                    parent::save($request, $response);
                }
                
                $this->connection->commit();
            }
            catch (\Exception $e) {
                var_dump($e);
                die;
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
                $this->connection->rollBack();
            }
        }
    }
    
    public function saveSenha(DTO\Request $request, DTO\Response $response) {
        $row                   = $request->getRow();
        $row["CDSENHAOPERWEB"] = md5($row["CDSENHAOPERWEB"]);
        $response->addDataSet(new DataSet("DADOS", array(
            "CDSENHAOPERWEB" => $row["CDSENHAOPERWEB"]
        )));
        parent::save($request, $response);
        
    }
    
    public function deleteOrg(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName      = 'organOper';
        $this->parameterBagColumns = array();
        parent::delete($request, $response);
    }
    
    public function getMaxOperCode($row, $response) {
        
        
        $params     = array(
            'NRORG' => $row['NRORG']
        );
        $cdoperador = $this->operadorPrincipalService->getMaxOperCode($params);
        $cdoperador = str_pad("" . $cdoperador . "", 12, "0", STR_PAD_LEFT);
        return $cdoperador;
        
        
    }
    
    public function criaParcnegocio($row, Response $response) {
        $params = array(
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORG" => $row["NRORG"],
            "NMOPERADOR" => $row["NMOPERADOR"],
            "NRINSCRICAOPARC" => $row["NRINSCRICAOPARC"],
            "IDATIVO" => 'S',
            "CDOPERINCLUSAO" => $row["CDOPERINCLUSAO"],
            "CDTIPOPARCPRINCIPAL" => "OPERADOR",
            "CDTIPOINSCRICAO" => $row["CDTIPOINSCRICAO"],
            "IDPESSOAFISICA" => 'S',
            "IDINSTITUICAO" => 'N',
            "IDPARCFUNDIDO" => 'N'
        );
        $this->operadorPrincipalService->criaParcnegocio($params);
        
        
    }
    
    public function criaParceria($row, Response $response, $cdtipoparceiro) {
        $params = array(
            "NRPARCERIA" => $row["NRPARCERIA"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORG" => $row["NRORG"],
            "CDTIPOPARCEIRO" => $cdtipoparceiro,
            "IDATIVO" => 'S',
            "CDOPERINCLUSAO" => $row["CDOPERINCLUSAO"]
        );
        $this->operadorPrincipalService->criaParceria($params);
        
        
    }
    
    public function criaRelacionaParc($row, Response $response) {
        $params = array(
            "NRRELACIONAPARC" => $row["NRRELACIONAPARC"],
            "NRORG" => $row["NRORG"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRTIPORELACIONA" => 25,
            "IDATIVO" => 'S',
            "CDOPERINCLUSAO" => $row["CDOPERINCLUSAO"],
            "CDCLIENTE" => $row["CDCLIENTE"]
        );
        
        $this->operadorPrincipalService->criaRelacionaParc($params);
        
        
    }
    
    public function criaInteclie($row, Response $response) {
        $params = array(
            "CDCLIENTE" => $row["CDCLIENTE"],
            "CDINTECLIE" => $row["CDINTECLIE"],
            "NMOPERADOR" => $row["NMOPERADOR"],
            "CDPAIS" => $row["CDPAIS"],
            "NRORG" => $row["NRORG"],
            "CDOPERINCLUSAO" => $row["CDOPERINCLUSAO"],
            "IDATIVO" => 'S',
            "NRRELACIONAPARC" => 63966
        );
        $this->operadorPrincipalService->criaInteclie($params);
        
        
    }
    
    public function criaOrganOper($row, Response $response) {
        $params = array(
            "NRORGANIZACAOOPER" => $row["NRORGANIZACAOOPER"],
            "NRORGTRAB" => $row["NRORGTRAB"],
            "CDOPERADOR" => $row["CDOPERADOR"],
            "CDOPERINCLUSAO" => $row["CDOPERINCLUSAO"],
            "NRORG" => $row["NRORG"],
            "IDATIVO" => 'S'
        );
        $this->operadorPrincipalService->criaOrganOper($params);
        
        
    }
    
    public function replicaFiliais($row, Response $response) {
        foreach ($row["FILIAIS"] as $filial) {
            $row["CDFILIAL"] = $filial;
            $params          = array(
                "CDOPERADOR" => $row["CDOPERADOR"],
                "CDFILIAL" => $row["CDFILIAL"],
                "IDEXIBAVISO" => 'S',
                "NRORG" => $row["NRORG"],
                "CDOPERINCLUSAO" => $row["CDOPERINCLUSAO"],
                "IDATIVO" => 'S'
            );
            $this->operadorPrincipalService->saveFilial($params);
        }
        
        
    }
    
    public function getOrganizacao(Row $row, Response $response) {
        
        
        $row = $row->getRow();
        $params = array(
            "NRORG" => $row["NRORG"]
        );
        $nmorg  = $this->operadorPrincipalService->getOrganizacao($params);
        $response->addDataSet(new DataSet("DADOS", $nmorg));
        
        
    }
    
    public function updateParcnegocio($row, $response) {
        $params = array(
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORG" => $row["NRORG"],
            "NMOPERADOR" => $row["NMOPERADOR"],
            "NRINSCRICAOPARC" => $row["NRINSCRICAOPARC"],
            "CDOPERULTATU" => $row["CDOPERULTATU"],
            "CDTIPOINSCRICAO" => $row["CDTIPOINSCRICAO"],
            "NRORGULTATU" => $row["NRORGULTATU"]
        );
        $this->operadorPrincipalService->updateParcnegocio($params);
        
        
        
    }
    
    public function validarUpdateCliente($row, $response) {
        if ($row["CDCLIENTE"] == null) {
            //Inativa Relações do Operador com o Cliente.
            $this->inativaTodosRelacParc($row, $response);
            $this->inativaTodosInteClie($row, $response);
            $this->setParceriaAtivaInativa($row, "N", $response);
            
        } else {
            //Verifica se existem relações, se não cria, se sim, ativa.
            if ($this->existeParceriaInterlocutor($row, $response) == 0) {
                
                $row["NRPARCERIA"] = NovoCodigo::geraCodigo($row["NRORG"], "PARCERIA", 1, 10, false, false);
                $this->criaParceria($row, $response, "INTERLOCUTOR");
                
            } else {
                
                $this->setParceriaAtivaInativa($row, "S", $response);
                
            }
            
            $this->inativaTodosRelacParc($row, $response);
            
            if ($this->existeRelacParc($row, $response) > 0) {
                
                $this->ativaRelacParc($row, $response);
                
            } else {
                
                $row["NRRELACIONAPARC"] = NovoCodigo::geraCodigo($row["NRORG"], 'RELACPARCNEG', 1, 10, false, false);
                $this->criaRelacionaParc($row, $response);
                
            }
            
            $this->inativaTodosInteClie($row, $response);
            
            if ($this->existeInteClie($row, $response) > 0) {
                
                $this->ativaInteClie($row, $response);
                
            } else {
                
                $row["CDINTECLIE"] = NovoCodigo::geraCodigo($row["NRORG"], 'INTECLIE' . $row["CDCLIENTE"], 1, 2, false, false);
                $this->criaInteclie($row, $response);
                
            }
        }
        
        
        
    }
    
    public function inativaTodosRelacParc($row, $response) {
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORGULTATU" => $row["NRORGULTATU"],
            "CDOPERULTATU" => $row["CDOPERULTATU"],
            "IDATIVO" => "N",
            "NRTIPORELACIONA" => 25
        );
        $this->operadorPrincipalService->inativaTodosRelacParc($params);
        
        
    }
    
    public function inativaTodosInteClie($row, $response) {
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORGULTATU" => $row["NRORGULTATU"],
            "CDOPERULTATU" => $row["CDOPERULTATU"],
            "IDATIVO" => "N",
            "NRTIPORELACIONA" => 25
        );
        
        $this->operadorPrincipalService->inativaTodosInteClie($params);
        
        
        
    }
    
    public function setParceriaAtivaInativa($row, $idativo, $response) {
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORGULTATU" => $row["NRORGULTATU"],
            "CDOPERULTATU" => $row["CDOPERULTATU"],
            "IDATIVO" => $idativo,
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );
        
        $this->operadorPrincipalService->setParceriaAtivaInativa($params);
        
        
        
    }
    
    public function existeParceriaInterlocutor($row, $response) {
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );
        return $this->operadorPrincipalService->existeParceriaInterlocutor($params);
        
        
    }
    
    public function existeRelacParc($row, $response) {
        
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRTIPORELACIONA" => 25,
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "CDCLIENTE" => $row["CDCLIENTE"]
        );
        return $this->operadorPrincipalService->existeRelacParc($params);
        
        
        
        
    }
    
    public function existeInteClie($row, $response) {
        
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRTIPORELACIONA" => 25,
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "CDCLIENTE" => $row["CDCLIENTE"]
        );
        return $this->operadorPrincipalService->existeInteClie($params);
        
        
    }
    
    public function ativaRelacParc($row, $response) {
        
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORGULTATU" => $row["NRORGULTATU"],
            "CDOPERULTATU" => $row["CDOPERULTATU"],
            "IDATIVO" => "S",
            "NRTIPORELACIONA" => 25,
            "CDCLIENTE" => $row["CDCLIENTE"]
        );
        $this->operadorPrincipalService->ativaRelacParc($params);
        
        
    }
    
    public function ativaInteClie($row, $response) {
        
        
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORGULTATU" => $row["NRORGULTATU"],
            "CDOPERULTATU" => $row["CDOPERULTATU"],
            "IDATIVO" => "S",
            "NRTIPORELACIONA" => 25,
            "CDCLIENTE" => $row["CDCLIENTE"]
        );
        $this->operadorPrincipalService->ativaInteClie($params);
        
        
    }
    
}
?>