<?php

namespace Teknisa\ACL\Services;

use Teknisa\ACL\Util\InstanceProvider;
use Teknisa\ACL\Util\AclUtilities;
use Teknisa\ACL\Util\Messages;
use Teknisa\ACL\Exception\CustomException;
use Teknisa\ACL\Repositories\Crud;
use Teknisa\Libs\Util\Utilities as LibUtilities;
use Zeedhi\Framework\Remote\FileUploader\Server;


class Operator {

    const ACTIVE_ID = 'S';
    const NOT_ACTIVE_ID = 'S';
    const USES_PASSWORD_CONTROL_POSITIVE = 'Y';

    const UPPERCASE_TYPE = 'UPPERCASE';
    const NUMERIC_TYPE   = 'NUMERIC';
    const SPECIAL_TYPE   = 'SPECIAL';

    private $fileServerFilesUrl;
    private $zhFileUploader;
    private $crud;

    public function __construct($fileServerFilesUrl, Server $zhFileUploader)
    {
        $this->fileServerFilesUrl = $fileServerFilesUrl;
        $this->zhFileUploader = $zhFileUploader;
        $this->crud = InstanceProvider::getRepository()->getCrud();
    }

    /*
     * Rotinas de CRUD
     */

    public function findOperator($conditions, $page, $pageSize, $businessPartnership = null)
    {
        $showFullOperator = AclUtilities::getValueInFilterDataSourceByName($conditions, 'SHOW_FULL_OPERATOR', true);
        $nrAccessProfile = InstanceProvider::getEnvironment()->getNrPerfil();
        $oAccCustomOrgId = $this->getOrganizationIdFromOAccCustom();
        $branches = null;
        $filterParcNegocio = '';
        $joinPerfAcessControl = $this->biuldJoinPerfAccessFilialControl($showFullOperator);
        $orgOperFilter = '';
        foreach ($conditions as $key => $condition) {
            if($condition['columnName'] === 'CDFILIAL') {
                $branches = "'".implode("','", $condition['value'])."'";
                unset($conditions[$key]);
            } else if($condition['columnName'] === 'NRORGOPER') {
                $orgOperFilter = $this->buildOrgOperFilter($condition['value'], $oAccCustomOrgId);
                unset($conditions[$key]);
            }
        }
        $idUtlPerfilfilial = InstanceProvider::getLibEnvironment()->getIdUtlPerfilFilial();
        if(!empty($businessPartnership) && ($idUtlPerfilfilial == 'N' || !$showFullOperator)){
            $businessPartnership = join(', ', $businessPartnership);
            $filterParcNegocio = " AND (NRPARCNEGOCIO NOT IN ($businessPartnership) OR NRPARCNEGOCIO IS NULL)";
        }
        $filters = AclUtilities::createQueryFiltered($conditions);
        if(!is_null($branches)){
            $filters .= $this->buildFilialFilter($branches);
        } else {
            $filters .= $this->biuldRelatedFilialFilter($showFullOperator);
            $joinPerfAcessControl .= $this->biuldJoinPerfAccessControl($showFullOperator);
        }
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;

        if(($idUtlPerfilfilial && $idUtlPerfilfilial != 'N')  && $showFullOperator){
            $modeParametrization = $idUtlPerfilfilial == 'P';
            $data = InstanceProvider::getRepository()->getOperator()->findOperatorsBranchProfile($organizationId, $filters, $limit, $offset, $filterParcNegocio, $joinPerfAcessControl, $nrAccessProfile, $modeParametrization, $oAccCustomOrgId, $orgOperFilter);
        } else {
            $data = InstanceProvider::getRepository()->getOperator()->findOperators($organizationId, $filters, $limit, $offset, $filterParcNegocio, $oAccCustomOrgId, $orgOperFilter);
        }
        return $data;
    }

    public function findOperatorOrgOper($cdoperador)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $oAccCustomOrgId = $this->getOrganizationIdFromOAccCustom();

        $data = InstanceProvider::getRepository()->getOperator()->findOperatorOrgOper($cdoperador, $organizationId);
        if(empty($data)) {
            $nrorg = empty($oAccCustomOrgId) ? $organizationId : $oAccCustomOrgId;
            return array($nrorg);
        }

        return array_map(function($register) {
            return $register['NRORGOPER'];
        }, $data);
    }

    public function findAccessHour($cdoperador)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $data = InstanceProvider::getRepository()->getOperator()->findAccessHour($cdoperador, $organizationId);
        if(empty($data)) {
            return array( array(
                'OPERSEMRESTRICAO' => 'S',
                'DIASEMANA' => array(),
                'HORACESSOINI' => '',
                'HORACESSOFIM' => '',
            ));
        } else {
            foreach($data as &$register) {
                $register['DIASEMANA'] = explode(',', $register['DIASEMANA']);
            }
        }
        return $data;
    }

    public function saveAccessHour($cdoperador, $hrIni, $hrFim, $weekDays, $accessHourId, $notRest) {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
            $repository     = InstanceProvider::getRepository()->getOperator();
            $newCode        = InstanceProvider::getNewCode();

            if($notRest == 'S') {
                $repository->deleteAccessHourDay($accessHourId, $organizationId);
                $entityManager->flush();
                $repository->deleteAccessHour($cdoperador, $organizationId);
            } else {
                if(empty($accessHourId)) {
                    $accessHourId = InstanceProvider::getNewCode()->getCodeByTableName('HORACESSOPER', $organizationId, 1, 10, true);
                    $repository->insertAccessHour($cdoperador, $hrIni, $hrFim, $accessHourId, $organizationId);
                } else {
                    $repository->updateAccessHour($cdoperador, $hrIni, $hrFim, $accessHourId, $organizationId);
                }
                $entityManager->flush();

                $repository->deleteAccessHourDay($accessHourId, $organizationId);
                $entityManager->flush();
                foreach($weekDays as $day) {
                    $id = InstanceProvider::getNewCode()->getCodeByTableName('HORACESSOPERDIA', $organizationId, 1, 10, true);
                    $repository->insertAccessHourDay($accessHourId, $id, $day, $organizationId);
                }
            }

            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function insertMultipleAccessHour($operators, $hrIni, $hrFim, $weekDays) {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
            $repository     = InstanceProvider::getRepository()->getOperator();
            $newCode        = InstanceProvider::getNewCode();

            foreach($operators as $cdoperador) {
                $accessHour = InstanceProvider::getRepository()->getOperator()->findAccessHour($cdoperador, $organizationId);
                if(empty($accessHour)) {
                    $accessHourId = InstanceProvider::getNewCode()->getCodeByTableName('HORACESSOPER', $organizationId, 1, 10, true);
                    $repository->insertAccessHour($cdoperador, $hrIni, $hrFim, $accessHourId, $organizationId);
                } else {
                    $accessHourId = $accessHour[0]['ID'];
                    $repository->updateAccessHour($cdoperador, $hrIni, $hrFim, $accessHourId, $organizationId);
                }
                $entityManager->flush();

                $repository->deleteAccessHourDay($accessHourId, $organizationId);
                $entityManager->flush();
                foreach($weekDays as $day) {
                    $id = InstanceProvider::getNewCode()->getCodeByTableName('HORACESSOPERDIA', $organizationId, 1, 10, true);
                    $repository->insertAccessHourDay($accessHourId, $id, $day, $organizationId);
                }
            }

            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    /*Filter FILIOPER */
    private function buildFilialFilter($branches)
    {
        $branches = explode(',', $branches);
        $inQueryString = AclUtilities::fixBugWhereINQuery($branches, 'CDFILIAL');
        return " AND (CDOPERADOR IN (SELECT CDOPERADOR FROM FILIOPER WHERE {$inQueryString}))";
    }

    /*Filter NRORGOPER */
    private function buildOrgOperFilter($orgOperFilter, $oAccCustomOrgId)
    {
        $organizationId = !empty($oAccCustomOrgId) ? $oAccCustomOrgId : InstanceProvider::getEnvironment()->getOrganizationId();
        $notFoundOrgOperatorsFilter = '';
        $isPrimary = false;
        foreach($orgOperFilter as $idx => $filterId) {
            if($filterId == 'XXXX') {
                unset($orgOperFilter[$idx]);
                $fullOrgOper = self::findOrgOper($organizationId, $isPrimary, null, true);
                $fullOrgOper = AclUtilities::getArrayFromAssociativeArray($fullOrgOper, 'NRORG');
                $notInQueryString = AclUtilities::fixBugWhereINQuery($fullOrgOper, 'ORL.NRORGOPER', true);
                $notFoundOrgOperatorsFilter = " OR EXISTS (SELECT ID 
                                                             FROM ORGRELACOPER ORL 
                                                            WHERE ORL.CDOPERADOR = OPER.CDOPERADOR 
                                                              AND $notInQueryString
                                                              AND ORL.NRORG = $organizationId)";
            }
        }
        $inQueryString = AclUtilities::fixBugWhereINQuery($orgOperFilter, 'ORL.NRORGOPER');
        return " AND (EXISTS (SELECT ID 
                                FROM ORGRELACOPER ORL 
                               WHERE ORL.CDOPERADOR = OPER.CDOPERADOR 
                                 AND $inQueryString
                                 AND ORL.NRORG = $organizationId)
                              $notFoundOrgOperatorsFilter
                          OR (OPER.NRORGOPER = 1)
                     )";
    }

    private function biuldRelatedFilialFilter($showFullOperator){
        $isprimary = false;
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $operatorCode = InstanceProvider::getEnvironment()->getCurrentUserId();
        self::findOrgOper($organizationId, $isprimary, $showFullOperator);
        $idUtlPerfilfilial = InstanceProvider::getLibEnvironment()->getIdUtlPerfilFilial();
        if(!$isprimary && ($idUtlPerfilfilial && $idUtlPerfilfilial != 'N')) {
            return " AND EXISTS (SELECT FP.CDFILIAL 
                                   FROM FILIOPER FP
                                   JOIN PERFACESSFILI PA ON PA.CDFILIAL = FP.CDFILIAL
                                                        AND PA.CDOPERADOR = '$operatorCode')";
        } else {
            return '';
        }
    }

    private function biuldJoinPerfAccessFilialControl($showFullOperator){
        $operatorCode = InstanceProvider::getEnvironment()->getCurrentUserId();
        $idUtlPerfilfilial = InstanceProvider::getLibEnvironment()->getIdUtlPerfilFilial();
        $return = '';
        if(($idUtlPerfilfilial && $idUtlPerfilfilial != 'N') && $showFullOperator){
            $return .=  <<<SQL
                                  JOIN (SELECT DISTINCT O.CDOPERADOR
                                            FROM OPERADOR O 
                                            JOIN FILIOPER FIOP ON (FIOP.CDOPERADOR = O.CDOPERADOR )
                                            JOIN FILIOPER FIOP2 ON (FIOP2.CDOPERADOR = '$operatorCode' AND FIOP2.CDFILIAL = FIOP.CDFILIAL )
                                            JOIN PERFACESSFILI PEFI ON ( PEFI.CDFILIAL = FIOP.CDFILIAL AND PEFI.CDOPERADOR = O.CDOPERADOR)
                                            JOIN PERFACESSCONTROL PFC ON (PFC.NRPERFILACESSO = PEFI.NRPERFILACESSO AND PFC.NRPERFACESSCONT = (SELECT CDPERFILSELECIONADO FROM OPERADOR WHERE CDOPERADOR = '$operatorCode'))
                                            UNION ALL
                                            SELECT DISTINCT O.CDOPERADOR
                                            FROM OPERADOR O 
                                            JOIN FILIOPER FIOP ON (FIOP.CDOPERADOR = O.CDOPERADOR )
                                            JOIN FILIOPER FIOP2 ON (FIOP2.CDOPERADOR = '$operatorCode' AND FIOP2.CDFILIAL = FIOP.CDFILIAL )
                                            LEFT JOIN PERFACESSFILI PEFI ON ( PEFI.CDFILIAL = FIOP.CDFILIAL AND PEFI.CDOPERADOR = O.CDOPERADOR)
                                            WHERE (PEFI.CDOPERADOR IS NULL)
                                ) X2 ON X2.CDOPERADOR = OPER.CDOPERADOR
SQL;
        }
        return $return;
    }

    private function biuldJoinPerfAccessControl($showFullOperator){
        $isprimary = false;
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $operatorCode = InstanceProvider::getEnvironment()->getCurrentUserId();
        self::findOrgOper($organizationId, $isprimary,$showFullOperator);
        $idUtlPerfilfilial = InstanceProvider::getLibEnvironment()->getIdUtlPerfilFilial();
        if(!$isprimary && ($idUtlPerfilfilial && $idUtlPerfilfilial != 'N')  ){
            return "     JOIN (SELECT
                         DISTINCT CDOPERADOR
                    FROM
                        OPERADOR 
                     WHERE CDOPERADOR NOT IN (SELECT
                            DISTINCT CDOPERADOR
                        FROM
                            PERFACESSFILI  
                        WHERE
                            NRPERFILACESSO  NOT IN (
                                SELECT
                                    NRPERFILACESSO
                                FROM
                                    PERFACESSCONTROL
                                WHERE
                                    NRPERFACESSCONT = (SELECT CDPERFILSELECIONADO FROM OPERADOR WHERE CDOPERADOR = '$operatorCode')
                            )
                            )) X ON X.CDOPERADOR = OPER.CDOPERADOR";
        } else {
            return '';
        }
    }

    public function findOrganization()
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findOrganization($organizationId);
    }

    public function findOrganizationZhuLog()
    {
        return InstanceProvider::getRepository()->getOperator()->findOrganizationZhuLog();
    }

    public function findAccessProfileByOperator($businessPartnership)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findAccessProfileByOperator($organizationId, $businessPartnership);
    }

    public function findAccessProfileByControlProfile($operador, $operadorcont, $cdfilial)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findAccessProfileByControlProfile($organizationId, $operador, $operadorcont, $cdfilial);
    }

    public function checkIfIsAdm($accessProfiles)
    {
        $isAdm = false;
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $generalParam = InstanceProvider::getRepository()->getOperator()->getParamGeralByNrorg($organizationId);
        $IdsAccessProfiles =  InstanceProvider::getRepository()->getOperator()->findIdAdmAccessProfile($organizationId, $accessProfiles);
        if((isset($generalParam['IDUTCTRLACESWEB']) && $generalParam['IDUTCTRLACESWEB'] == 'N') || empty($generalParam['IDUTCTRLACESWEB'])) {
            $isAdm = true;
        } else {
            foreach($IdsAccessProfiles as $idAdm){
                if($idAdm['IDPERFILACESSOADM'] === 'S'){
                    $isAdm = true;
                }
            }
        }
        return $isAdm;
    }

    public function saveLogLib($operatorCode, $operatorName, $clientCode, $businessPartnership, $password, $labels) {
        $valores = [$labels['label'], $operatorCode, $operatorName, $clientCode, $businessPartnership, '*************'];
        $nomes = ["Tela", $labels['CDOPERADOR'], $labels['NMOPERADOR'], $labels['CDCLIENTE'], $labels['NRPARCNEGOCIO'], $labels['NEW_PASSWORD']];
        $descricao = AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($operatorCode, $descricao, '1', 'OPERADOR', '/operator/save');
    }

    public function saveLogLibCdFilial($operatorCode, $operatorName, $clientCode, $businessPartnership, $password, $nrestruttrab, $idtpoperador, $cdFilial, $labels) {
        $valores = [$labels['label'], $operatorCode, $operatorName, $clientCode, $businessPartnership, '*************', $nrestruttrab, $idtpoperador, $cdFilial];
        $nomes = ["Tela", $labels['CDOPERADOR'], $labels['NMOPERADOR'], $labels['CDCLIENTE'], $labels['NRPARCNEGOCIO'], $labels['NEW_PASSWORD'], $labels['NMESTRUTURAM'], $labels['IDTPOPERADOR'], $labels['CDFILIAL']];
        $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($operatorCode, $descricao, '1', 'OPERADOR', '/operator/save');
    }

    public function editLogLib($operatorCode, $operatorName, $clientCode, $businessPartnership, $activeId, $accessName, 
    $operatorEmail, $countryCode, $nrestruttrab, $idtpoperador, $labels) {
        $operatorRepository = InstanceProvider::getRepository()->getOperator();
        $operatorAntigo = $operatorRepository->findValAntigoOperators($operatorCode);
        $arrayValores = ['operator'];
        $arrayNomes = ['Tela'];
        if ($operatorAntigo[0]['NMOPERADOR'] !== $operatorName) {
            array_push($arrayValores, $operatorName);
            array_push($arrayNomes, $labels['NMOPERADOR']);
        }
        if ($operatorAntigo[0]['CDCLIENTE'] !== $clientCode) {
            array_push($arrayValores, $clientCode);
            array_push($arrayNomes, $labels['CDCLIENTE']);
        }
        if ($operatorAntigo[0]['NRPARCNEGOCIO'] !== $businessPartnership) {
            array_push($arrayValores, $businessPartnership);
            array_push($arrayNomes, $labels['NRPARCNEGOCIO']);
        }
        if ($operatorAntigo[0]['IDATIVO'] !== $activeId) {
            array_push($arrayValores, $activeId);
            array_push($arrayNomes, $labels['IDATIVO']);
        }
        if ($operatorAntigo[0]['NMACESSO'] !== $accessName) {
            array_push($arrayValores, $accessName);
            array_push($arrayNomes, $labels['NMACESSO']);
        }
        if ($operatorAntigo[0]['DSEMAILOPER'] !== $operatorEmail) {
            array_push($arrayValores, $operatorEmail);
            array_push($arrayNomes, $labels['DSEMAILOPER']);
        }
        if ($operatorAntigo[0]['CDPAIS'] !== $countryCode) {
            array_push($arrayValores, $countryCode);
            array_push($arrayNomes, $labels['CDPAIS']);
        }
        if ($operatorAntigo[0]['NRESTRUTTRAB'] !== $nrestruttrab) {
            array_push($arrayValores, $nrestruttrab);
            array_push($arrayNomes, $labels['NMESTRUTURAM']);
        }
        if ($operatorAntigo[0]['IDTPOPERADOR'] !== $idtpoperador) {
            array_push($arrayValores, $idtpoperador);
            array_push($arrayNomes, $labels['IDTPOPERADOR']);
        }
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($operatorCode, $descricao, '2', 'OPERADOR', '/operator/save');
    }

    public function saveOperator(
        $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
        $cpf, $countryName, $countryCode, $password, $confirmPassword, $businessPartnership,
        $clientCode, $accessOperatorId,
        $selectedOrganizationNumber, $selectedBranchId, $selectedProfile, $inscriptionTypeCode,
        $operatorInclusionCode, $idopermaster, $nrestruttrab, $idtpoperador, $administrator, $branches, $nrOrgOper, $cdFilial, $showFullOperator,  $labels)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $libEnvironment = InstanceProvider::getLibEnvironment();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            $operatorInclusionCode = $libEnvironment->getCurrentUserId();
            $operatorRepository = InstanceProvider::getRepository()->getOperator();

            $workOrganizationId = $organizationId;
            $organizationInclusion = $organizationId;
            $nrParcNegovio = null;
            if(!$operatorCode){
                $colSize = 12 - strlen($organizationId);
                $newCode = InstanceProvider::getNewCode()->getCodeByTableName('OPERADOR'.$organizationId, $organizationId, 1, 10, true);


                if(strlen($organizationId . $newCode) >= 12){
                    $newCode = substr($newCode, strlen($organizationId));
                }
                $operatorCode = $organizationId . $newCode;
                $operatorCode = str_pad($operatorCode, 12, 0, STR_PAD_LEFT);
                $operatorEntity = InstanceProvider::getRepository()->getOperator()->findBy(array('CDOPERADOR' => $operatorCode));
                do{
                    $operatorCode +=   1;
                    $operatorCode = str_pad($operatorCode, 12, 0, STR_PAD_LEFT);
                    $operatorEntity = InstanceProvider::getRepository()->getOperator()->findBy(array('CDOPERADOR' => $operatorCode));
                }
                while(!empty($operatorEntity));
            }

            $this->operatorSaveValidation($operatorCode, $operatorEmail, $organizationId, $password, $confirmPassword);
            $parcNegocios = $this->operatorSaveValidationCpf($cpf, $organizationId);

            if($businessPartnership){
                $operators = $operatorRepository->findBy(array('NRPARCNEGOCIO' => $businessPartnership), $organizationId);
                if(!$operators){
                    throw new CustomException('ERR_INVALID_PARCNEGOCIO');
                }
            } else {
                if(empty($parcNegocios)) {
                    $businessPartnership = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', $organizationId);
                    $this->createBusinessPartnership($businessPartnership, $organizationId, $operatorName, $cpf, $activeId, $operatorInclusionCode, $inscriptionTypeCode);
                    $entityManager->flush();
                    $valores = [$labels['label'], $businessPartnership];
                    $nomes = ["Tela", $labels['NRPARCNEGOCIO']];
                    $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
                    InstanceProvider::getLibLog()->logRegister($businessPartnership, $descricao, '1', 'PARCNEGOCIO', '/operator/save');
                } else {
                    $businessPartnership = $parcNegocios[0]['NRPARCNEGOCIO'];
                }
            }

            $this->createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, 'OPERADOR');
            $entityManager->flush();

            if($clientCode != null){
                $partnershipRelationship = InstanceProvider::getNewCode()->getCodeByTableName('RELACPARCNEG', $organizationId);
                $this->createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, 'INTERLOCUTOR');
            }

            $password = md5($password);
            $idUtlPerfilfilial = $libEnvironment->getIdUtlPerfilFilial();
            if(($idUtlPerfilfilial && $idUtlPerfilfilial != 'N')  && $showFullOperator){

                $nrOrgOper = array($organizationId);

                $operatorRepository->saveOperatorWithCdFilial(
                    $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
                    $countryCode, $businessPartnership, $clientCode, $organizationId,
                    $accessOperatorId, $selectedProfile, $organizationInclusion,
                    $operatorInclusionCode, $password, $idopermaster, $nrestruttrab, $idtpoperador, $cdFilial
                );
                $this->saveLogLibCdFilial($operatorCode, $operatorName, $clientCode, $businessPartnership, $password, $nrestruttrab, $idtpoperador, $cdFilial, $labels);
            } else {
                $operatorRepository->saveOperator(
                    $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
                    $countryCode, $businessPartnership, $clientCode, $organizationId,
                    $accessOperatorId, $selectedProfile, $organizationInclusion,
                    $operatorInclusionCode, $password, $idopermaster, $nrestruttrab, $idtpoperador
                );
                $this->saveLogLib($operatorCode, $operatorName, $clientCode, $businessPartnership, $password, $labels);
            }
            $entityManager->flush();

            $this->createRelationshipOragnizationOperator($workOrganizationId, $operatorCode, $organizationId, $operatorInclusionCode);
            if(!empty($branches)){
                foreach ($branches as $branch) {
                    $operatorRepository->importBranches($operatorCode, $organizationId, $branch, $operatorInclusionCode);
                }
            }
            
            $operatorRepository->updateBusinessPartnershipOperator($businessPartnership, $operatorCode, $organizationId);
            $this->saveOrgOper($operatorCode, $nrOrgOper, $labels);
            $entityManager->flush();
            
            $connection->commit();
            return array("CDOPERADOR" => $operatorCode, "NRPARCNEGOCIO"  => $businessPartnership);
        } catch (\Exception $e){
            $connection->rollBack();
            if (strpos($e->getMessage(), 'UN_OPERADOR') !== false && strpos($e->getMessage(), 'OPERADOR_SAAS') !== false) {
                $msg = 'ERR_EXISTS_EMAIL_SAAS';
                throw new CustomException($msg);
            } else {
                throw $e;
            }
        }
    }

    public function editOperator(
        $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
        $cpf, $countryName, $countryCode, $businessPartnership,
        $clientCode, $accessOperatorId,
        $selectedOrganizationNumber, $selectedBranchId, $selectedProfile, $inscriptionTypeCode,
        $operatorInclusionCode, $oldCpf, $oldClientCode, $oldOperatorName, $idopermaster, $nrestruttrab, $idtpoperador, $nrOrgOper, $labels)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $operatorRepository = InstanceProvider::getRepository()->getOperator();
        $libEnvironment = InstanceProvider::getLibEnvironment();

        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            $userId = $libEnvironment->getCurrentUserId();
            $workOrganizationId = $organizationId;

            $this->operatorEditValidation($operatorCode, $cpf, $operatorEmail, $organizationId, $businessPartnership);
            $parcNegocios = $this->operatorEditValidationCpf($cpf, $organizationId, $businessPartnership);

            if(!$businessPartnership){
                if(empty($parcNegocios)) {
                    $businessPartnership = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', $organizationId);
                    $this->createBusinessPartnership($businessPartnership, $organizationId, $operatorName, $cpf, $activeId, $operatorInclusionCode, $inscriptionTypeCode);
                    $entityManager->flush();
                    $valores = [$labels['label'], $businessPartnership];
                    $nomes = ["Tela", $labels['NRPARCNEGOCIO']];
                    $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
                    InstanceProvider::getLibLog()->logRegister($businessPartnership, $descricao, '1', 'PARCNEGOCIO', '/operator/save');
                } else {
                    $businessPartnership = $parcNegocios[0]['NRPARCNEGOCIO'];
                }

                $this->createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, 'OPERADOR');
                $entityManager->flush();

            } else if ($cpf != $oldCpf || $operatorName != $oldOperatorName){
                $operatorRepository->updateBusinessPartner($cpf, $inscriptionTypeCode, $operatorName, $organizationId, $userId, $businessPartnership);
                $entityManager->flush();
                if ($oldCpf !== $cpf) {
                    $valores = [$labels['label'], $cpf];
                    $nomes = ["Tela", $labels['NRINSCRICAOPARC']];
                    $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
                    InstanceProvider::getLibLog()->logRegister($businessPartnership, $descricao, '2', 'PARCNEGOCIO', '/operator/save');
                }
            }

            if($oldClientCode != $clientCode){
                if($clientCode == null){
                    $operatorRepository->inactiveAllPartnershipRelationships($userId, $organizationId, $businessPartnership, self::NOT_ACTIVE_ID);
                    $entityManager->flush();
                    $operatorRepository->inactiveAllClientPartners($userId, $organizationId, $businessPartnership);
                    $entityManager->flush();
                    $operatorRepository->setActiveInactivePartnership($userId, $organizationId, $businessPartnership, self::NOT_ACTIVE_ID);
                    $entityManager->flush();
                } else {
                    if($operatorRepository->isThereInterlocutorPartnership($organizationId, $businessPartnership) == 0){
                        $this->createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, 'INTERLOCUTOR');
                        $entityManager->flush();
                    } else {
                        $operatorRepository->setActiveInactivePartnership($userId, $organizationId, $businessPartnership, self::ACTIVE_ID);
                        $entityManager->flush();
                    }

                    $operatorRepository->inactiveAllPartnershipRelationships($userId, $organizationId, $businessPartnership, self::NOT_ACTIVE_ID);
                    $entityManager->flush();

                    if($operatorRepository->isTherePartnerRelationship($organizationId, $businessPartnership, $clientCode) > 0){
                        $operatorRepository->activateClientPartner($organizationId, $businessPartnership, $clientCode, $userId, self::ACTIVE_ID);
                        $entityManager->flush();
                    }
                }
            }
            $this->editLogLib($operatorCode, $operatorName, $clientCode, $businessPartnership, $activeId, $accessName, 
            $operatorEmail, $countryCode, $nrestruttrab, $idtpoperador, $labels);
            $operatorRepository->editOperator(
                $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
                $countryCode, $clientCode, $organizationId, $workOrganizationId, $userId, $businessPartnership, $idopermaster, $nrestruttrab, $idtpoperador
            );

            $this->saveOrgOper($operatorCode, $nrOrgOper, $labels);
            $entityManager->flush();
            $connection->commit();

            return array("CDOPERADOR" => $operatorCode, "NRPARCNEGOCIO"  => $businessPartnership);
        } catch (\Exception $e){
            $connection->rollBack();
            if (strpos($e->getMessage(), 'UN_OPERADOR') !== false && strpos($e->getMessage(), 'OPERADOR_SAAS') !== false) {
                $msg = 'ERR_EXISTS_EMAIL_SAAS';
                throw new CustomException($msg);
            } else {
                throw $e;
            }
        }
    }

    private function saveOrgOper($operatorCode, $nrOrgOper, $labels) {
        $operRepository = InstanceProvider::getRepository()->getOperator();
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();

        $operRepository->deleteOrgRelacOper($operatorCode, $organizationId);
        InstanceProvider::getEntityManager()->flush();
        foreach($nrOrgOper as $org) {
            $id = InstanceProvider::getNewCode()->getCodeByTableName('ORGRELACOPER', $organizationId, 1, 10, true);
            $operRepository->insertOrgRelacOper($id, $org, $operatorCode, $organizationId);
            $valores = [$labels['label'], $operatorCode];
            $nomes = ["Tela", $labels['CDOPERADOR']];
            $descricao = AclUtilities::gerarDescricao($valores, $nomes);
            InstanceProvider::getLibLog()->logRegister($id, $descricao, '1', 'ORGRELACOPER', '/operator/save');
        }

        if(!empty($nrOrgOper)) {
            $operRepository->updateNrorgOper($organizationId, $nrOrgOper[0], $operatorCode);
            $operatorAntigo = $operRepository->findValAntigoOperators($operatorCode);
            if ($operatorAntigo[0]['NRORGOPER'] !== $nrOrgOper[0]) {
                $valores = [$labels['label'], $nrOrgOper[0]];
                $nomes = ["Tela", $labels['NRORGOPER']];
                $descricao = AclUtilities::gerarDescricao($valores, $nomes);
                InstanceProvider::getLibLog()->logRegister($operatorCode, $descricao, '2', 'OPERADOR', '/operator/save');
            }
        }
    }

    public function deleteOperator($operatorCode)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        InstanceProvider::getRepository()->getOperator()->deleteOperator($operatorCode, $organizationId);
    }

    /**
     * @param $conditions
     * @param $orderBy
     * @return mixed
     * @throws \Exception
     */
    public function findPartnerAccessProfile($conditions, $orderBy)
    {
        $filters = AclUtilities::createQueryFiltered($conditions);
        //If, for some reason, current operator doens't have a business partnership, fix the filter
        $filters .= $this->filterNullPartner($conditions);
        $orderByString = '';
        foreach($orderBy as $idx => $order) {
            if($idx === 'DTVALIDADE'){
                $orderByString .= ' ORDER BY TO_DATE(' . $idx . ') ' . $order . ' NULLS LAST';
            } else {
                $orderByString .= ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        $data = InstanceProvider::getRepository()->getOperator()->findPartnerAccessProfile($filters, $organizationId, $orderByString);
        foreach ($data as &$p){
            if(!$p['DTVALIDADE']){
                $p['DTVALIDADE'] = '';
            }
        }
        return $data;
    }

    public function findBranchAccessProfile($conditions, $orderBy)
    {
        $filters = AclUtilities::createQueryFiltered($conditions);
        //If, for some reason, current operator doens't have a business partnership, fix the filter
        $filters .= $this->filterNullPartner($conditions);
        $orderByString = '';
        foreach($orderBy as $idx => $order) {
            if($idx === 'DTVALIDADE'){
                $orderByString .= ' ORDER BY TO_DATE(' . $idx . ') ' . $order . ' NULLS LAST';
            } else {
                $orderByString .= ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        $data = InstanceProvider::getRepository()->getOperator()->findBranchAccessProfile($filters, $organizationId, $orderByString);
        foreach ($data as &$p){
            if(!$p['DTVALIDADE']){
                $p['DTVALIDADE'] = '';
            }
        }
        return $data;
    }

    private function filterNullPartner($conditions)
    {
        $nullFilter = '';
        foreach ($conditions as $filter) {
            if($filter['columnName'] === 'NRPARCNEGOCIO' && is_null($filter['value'])){
                $nullFilter = ' AND NRPARCNEGOCIO IS NULL';
            }
        }
        return $nullFilter;
    }

    public function saveAccessProfileByOperator($accessProfiles, $labels)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            foreach($accessProfiles as $accessProfile){
                $accessProfile['NRPERFACESSPARC']   = InstanceProvider::getNewCode()->getCodeByTableName('PERFACESSPARC', $organizationId);
                $accessProfile['NRORG']             = $organizationId;
                $accessProfile['NRORGINCLUSAO']     = $organizationId;
                $accessProfile['CDOPERINCLUSAO']    = $libEnvironment->getCurrentUserId();
                InstanceProvider::getRepository()->getOperator()->saveAccessProfileByOperator($accessProfile);
                $operatorByPerfil = InstanceProvider::getRepository()->getOperator()->findOperadorByPerfil($accessProfile['NRPARCNEGOCIO']);
                $valores = [$labels['label'], $operatorByPerfil[0]['CDOPERADOR'], $operatorByPerfil[0]['NMOPERADOR'], $accessProfile['NMPERFILACESSO'], $accessProfile['NRPARCNEGOCIO'], $accessProfile['NRPERFILACESSO'], $accessProfile['DTVALIDADE']];
                $nomes = ["Tela", 'Cod.Operador', 'Operador', 'Perfil', 'NRPARCNEGOCIO', 'NRPERFILACESSO', $labels['DTVALIDADE']];
                $descricao = AclUtilities::gerarDescricao($valores, $nomes);
                InstanceProvider::getLibLog()->logRegister($accessProfile['NRPERFACESSPARC'], $descricao, '1', 'PERFACESSPARC', '/accessProfileByOperator/save');
            }
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveAccessProfileByControlProfile($accessProfiles, $labels)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            foreach($accessProfiles as $accessProfile){
                $accessProfile['NRORG']             = $organizationId;
                $accessProfile['NRORGINCLUSAO']     = $organizationId;
                $accessProfile['CDOPERINCLUSAO']    = $libEnvironment->getCurrentUserId();
                $pkPerfAcessFili =  $accessProfile['CDFILIAL'].'_'.$accessProfile['CDOPERADOR'].'_'.$accessProfile['NRPERFILACESSO'];
                InstanceProvider::getRepository()->getOperator()->saveAccessProfileByControlProfile($accessProfile);
                $valores = [$labels['label'], $accessProfile['DTVALIDADE']];
                $nomes = ["Tela", $labels['DTVALIDADE']];
                $descricao = AclUtilities::gerarDescricao($valores, $nomes);
                InstanceProvider::getLibLog()->logRegister($pkPerfAcessFili, $descricao, '1', 'PERFACESSFILI', '/accessProfileByControlProfile/save');
            }
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveRelatedBranchByOperator($relatedBranches, $labels)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            foreach($relatedBranches as $relatedBranch){
                $relatedBranch['NRORG']             = $organizationId;
                $relatedBranch['NRORGINCLUSAO']     = $organizationId;
                $relatedBranch['CDOPERINCLUSAO']    = $libEnvironment->getCurrentUserId();
                InstanceProvider::getRepository()->getOperator()->saveRelatedBranchByOperator($relatedBranch);
                $pkFiliOper =  $relatedBranch['CDOPERADOR'].'_'.$relatedBranch['CDFILIAL'];
                $valores = [$labels['label'], $relatedBranch['CDOPERADOR'], $relatedBranch['CDFILIAL'], $relatedBranch['NMESTADO'], $relatedBranch['NMFILIAL'], $relatedBranch['NMMUNICIPIO'], $relatedBranch['NMPAIS'], $relatedBranch['NMREGIAO'], $relatedBranch['DTVALIDADE']];
                $nomes = ["Tela", 'CDOPERADOR', $labels['CDFILIAL'], $labels['NMESTADO'], $labels['NMFILIAL'], $labels['NMMUNICIPIO'], $labels['NMPAIS'], $labels['NMREGIAO'],'DTVALIDADE'];
                $descricao = AclUtilities::gerarDescricao($valores, $nomes);
                InstanceProvider::getLibLog()->logRegister($pkFiliOper, $descricao, '1', 'FILIOPER', '/relatedBranchByOperator/save');
            }
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteRelatedBranch($cdfilial, $cdoperador)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        InstanceProvider::getRepository()->getOperator()->deleteRelatedBranch($cdfilial, $cdoperador, $organizationId);
    }

    public function editRelatedBranchByOperator($cdfilial, $cdoperador, $activeId, $dtValidade, $labels)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        $oldValuesFiliOper = InstanceProvider::getRepository()->getOperator()->findOldValFiliOper($cdfilial, $cdoperador);
        $pkFiliOper =  $cdoperador.'_'.$cdfilial;
        $arrayValores = ['operator'];
        $arrayNomes = ['Tela'];
        if ($oldValuesFiliOper[0]['DTVALIDADE'] !== $dtValidade) {
            array_push($arrayValores, $dtValidade);
            array_push($arrayNomes, $labels['DTVALIDADE']);
        }
        if ($oldValuesFiliOper[0]['IDATIVO'] !== $activeId) {
            array_push($arrayValores, $activeId);
            array_push($arrayNomes, $labels['IDATIVO']);
        }
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($pkFiliOper, $descricao, '2', 'FILIOPER', '/relatedBranchByOperator/save');
        InstanceProvider::getRepository()->getOperator()->editRelatedBranchByOperator($cdfilial, $cdoperador, $activeId, $organizationId, $dtValidade);
    }

    public function saveRelatedOrganizationByOperator($relatedOrganizations)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            foreach($relatedOrganizations as $relatedOrganization){
                $relatedOrganization['NRORGANIZACAOOPER'] = InstanceProvider::getNewCode()->getCodeByTableName('ORGANIZACAOOPER', $organizationId);
                $relatedOrganization['NRORG']             = $organizationId;
                $relatedOrganization['NRORGREL']          = $organizationId;
                $relatedOrganization['NRORGINCLUSAO']     = $organizationId;
                $relatedOrganization['CDOPERINCLUSAO']    = $libEnvironment->getCurrentUserId();
                InstanceProvider::getRepository()->getOperator()->saveRelatedOrganizationByOperator($relatedOrganization);
            }
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteRelatedOrganization($nrorgrel, $cdoperador)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        InstanceProvider::getRepository()->getOperator()->deleteRelatedOrganization($nrorgrel, $cdoperador, $organizationId);
    }

    public function editRelatedOrganizationByOperator($nrorgrel, $cdoperador, $activeId)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        InstanceProvider::getRepository()->getOperator()->editRelatedOrganizationByOperator($nrorgrel, $cdoperador, $activeId, $organizationId);
    }

    public function deletePartnerAccessProfile($partnerAccessProfileNumber, $labels, $nmperfacess, $nrparcnegocio)
    {
        $operatorByPerfil = InstanceProvider::getRepository()->getOperator()->findOperadorByPerfil($nrparcnegocio);
        $valores = [$labels['label'], $operatorByPerfil[0]['CDOPERADOR'], $operatorByPerfil[0]['NMOPERADOR'], $nmperfacess, $partnerAccessProfileNumber];
        $nomes = ["Tela", 'Cod.Operador', 'Operador', $labels['NMPERFILACESSO'], 'NRPERFACESSPARC'];
        $descricao = AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($partnerAccessProfileNumber, $descricao, '3', 'PERFACESSPARC', '/partnerAccessProfile/remove');
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        InstanceProvider::getRepository()->getOperator()->deletePartnerAccessProfile($partnerAccessProfileNumber, $organizationId);
    }

    public function deleteBranchAccessProfile($branchAccessProfileNumber, $operator, $branch, $labels)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        $operatorData = InstanceProvider::getRepository()->getOperator()->getOperatorData($operator, $organizationId);
        InstanceProvider::getRepository()->getOperator()->deleteBranchAccessProfile($branchAccessProfileNumber, $operator, $branch, $organizationId);
        $pkPerfAcessFili = $branch.'_'.$operator.'_'.$branchAccessProfileNumber;
        $valores = [$labels['label'], $branch, $operator, $branchAccessProfileNumber];
        $nomes = ["Tela", $labels['CDFILIAL'], $labels['CDOPERADOR'], $labels['NRPERFILACESSO']];
        $descricao = AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($pkPerfAcessFili, $descricao, '3', 'PERFACESSFILI', '/branchAccessProfile/remove');
        if($operatorData['CDPERFILSELECIONADO'] == $branchAccessProfileNumber){
            $arrayValores = ['operator'];
            $arrayNomes = ['Tela'];
            if ($operatorData['CDPERFILSELECIONADO'] !== null) {
                array_push($arrayValores, null);
                array_push($arrayNomes, 'CDPERFILSELECIONADO');
            }
            if ($operatorData['CDFILIALSELECIONADA'] !== null) {
                array_push($arrayValores, null);
                array_push($arrayNomes, 'CDFILIALSELECIONADA');
            }
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($operator, $descricao, '2', 'OPERADOR', '/branchAccessProfile/remove');
            InstanceProvider::getRepository()->getOperator()->deleteRelatedBranchAndProfile($operator, $organizationId);
        }

    }

    /**
     * @param $partnerAccessProfileNumber
     * @param $activeId
     * @throws \Exception
     */
    public function editAccessProfileByOperator($partnerAccessProfileNumber, $activeId, $dtValidade, $labels)
    {
        $operatorRepository = InstanceProvider::getRepository()->getOperator();
        $perfAcessAntigo = $operatorRepository->findValAntigoPerfAcess($partnerAccessProfileNumber);
        $arrayValores = ['operator'];
        $arrayNomes = ['Tela'];
        if ($perfAcessAntigo[0]['IDATIVO'] !== $activeId) {
            array_push($arrayValores, $activeId);
            array_push($arrayNomes, $labels['IDATIVO']);
        }
        if ($perfAcessAntigo[0]['DTVALIDADE'] !== $dtValidade) {
            array_push($arrayValores, $dtValidade);
            array_push($arrayNomes, $labels['DTVALIDADE']);
        }
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($partnerAccessProfileNumber, $descricao, '2', 'PERFACESSPARC', '/accessProfileByOperator/save');
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        InstanceProvider::getRepository()->getOperator()->editAccessProfileByOperator($partnerAccessProfileNumber, $activeId, $organizationId, $dtValidade);
    }

    public function editAccessProfileByControlProfile($accessProfileNumber,  $operador, $unidade, $activeId, $dtValidade, $labels)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        $oldValuePerfAcessFili = InstanceProvider::getRepository()->getOperator()->findOldValPerfAcessFili($unidade,  $operador, $accessProfileNumber);
        $pkPerfAcessFili = $unidade.'_'.$operador.'_'.$accessProfileNumber;
        $arrayValores = ['operator'];
        $arrayNomes = ['Tela'];
        if ($oldValuePerfAcessFili[0]['IDATIVO'] !== $activeId) {
            array_push($arrayValores, $activeId);
            array_push($arrayNomes, $labels['IDATIVO']);
        }
        if ($oldValuePerfAcessFili[0]['DTVALIDADE'] !== $dtValidade) {
            array_push($arrayValores, $dtValidade);
            array_push($arrayNomes, $labels['DTVALIDADE']);
        }
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($pkPerfAcessFili, $descricao, '2', 'PERFACESSFILI', '/accessProfileByControlProfile/save');
        InstanceProvider::getRepository()->getOperator()->editAccessProfileByControlProfile($accessProfileNumber,  $operador, $unidade, $activeId, $organizationId, $dtValidade);
    }

    public function setBusinessPartnership($operatorName, $cpf, $activeId, $operatorInclusionCode, $inscriptionTypeCode, $operatorCode)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
            $businessPartnership = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', $organizationId);

            $this->createBusinessPartnership($businessPartnership, $organizationId, $operatorName, $cpf, $activeId, $operatorInclusionCode, $inscriptionTypeCode);
            $entityManager->flush();
            $this->createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, 'OPERADOR');
            $entityManager->flush();

            $table = 'OPERADOR';
            $operator =  $this->crud->findOne($table, '*',  "WHERE CDOPERADOR = '{$operatorCode}'", $organizationId);

            $fieldsValues = array(
                'NRPARCNEGOCIO'  => $businessPartnership,
            );
            $conditions = "CDOPERADOR = '{$operator['CDOPERADOR']}'";
            $this->crud->update($table, $fieldsValues, $conditions);
            $entityManager->flush();

            $connection->commit();

            return array(
                'NRPARCNEGOCIO'   => $businessPartnership,
                'NMPRINCIPALPARC' => $operatorName
            );
        } catch (Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    private function createBusinessPartnership($businessPartnership, $organizationId, $operatorName, $cpf,
        $activeId, $operatorInclusionCode, $inscriptionTypeCode)
    {
        InstanceProvider::getRepository()->getOperator()->createBusinessPartnership($businessPartnership, $organizationId, $operatorName, $cpf, $activeId, $operatorInclusionCode, $inscriptionTypeCode);
    }

    private function createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, $partnerTipeCode)
    {
        $partnerNumber = InstanceProvider::getNewCode()->getCodeByTableName('PARCERIA', $organizationId);
        $activeId = 'S';
        InstanceProvider::getRepository()
                        ->getOperator()
                        ->createPartnership($businessPartnership, $organizationId, $operatorInclusionCode,
                            $partnerTipeCode, $partnerNumber, $activeId);
    }

    public function createRelationshipOragnizationOperator($workOrganizationId, $operatorCode, $organizationId,
            $operatorInclusionCode)
    {
        $organizationOperator = InstanceProvider::getNewCode()->getCodeByTableName('ORGANIZACAOOPER', $organizationId);
        $activeId = 'S';
        InstanceProvider::getRepository()
                        ->getOperator()
                        ->createRelationshipOragnizationOperator($organizationOperator, $workOrganizationId, $operatorCode, $organizationId,
                            $operatorInclusionCode, $activeId);

    }

    public function operatorSaveValidation($operatorCode, $operatorEmail, $organizationId, $password, $confirmPassword)
    {
        $crudRepository = InstanceProvider::getRepository()->getCrud();
        $operatorRepository = InstanceProvider::getRepository()->getOperator();
        $msgError = '';
        $operator = $operatorRepository->findOperatorsByCode($operatorCode);
        if(!empty($operator)) {
            if($operator[0]['NRORG'] == $organizationId) {
                $msgError = 'ERR_EXISTS_OPERATOR_CODE';
            } else {
                $msgError = 'ERR_EXISTS_OPERATOR_CODE_DIFFERENT_ORGANIZATION';
            }
        }

        if($crudRepository->findCount("OPERADOR", "WHERE DSEMAILOPER = '{$operatorEmail}' AND NRORG = {$organizationId}") > 0){
            $msgError = 'ERR_EXISTS_EMAIL';
        }

        if($msgError){
            throw new CustomException($msgError);
        }

        $this->passwordValidate($password, $confirmPassword);
    }

    public function operatorSaveValidationCpf($cpf, $organizationId)
    {
        if($cpf){
            $parcNegocioRepository = InstanceProvider::getRepository()->getparcNegocio();
            $operatorRepository = InstanceProvider::getRepository()->getOperator();
            $cpfIsUsed = false;

            $parcNegocios = $parcNegocioRepository->findBy(array('NRINSCRICAOPARC' => $cpf), $organizationId);
            foreach ($parcNegocios as $pn) {
                $operators = $operatorRepository->findBy(array('NRPARCNEGOCIO' => $pn['NRPARCNEGOCIO']), $organizationId);
                if(!empty($operators)) {
                    $cpfIsUsed = true;
                    break;
                }
            }
            if($cpfIsUsed){
                throw new CustomException('ERR_EXISTS_CPF');
            }
            return $parcNegocios;
        }
        return [];
    }

    public function operatorEditValidation($operatorCode, $cpf, $operatorEmail, $organizationId, $businessPartnership)
    {
        $crudRepository = InstanceProvider::getRepository()->getCrud();
        $msgError = '';

        if($crudRepository->findCount("OPERADOR", "WHERE DSEMAILOPER = '{$operatorEmail}' AND NRORG = {$organizationId} AND CDOPERADOR != '{$operatorCode}'") > 0){
            $msgError .= 'ERR_EXISTS_EMAIL';
        }

        if($msgError){
            throw new CustomException($msgError);
        }
    }

    public function operatorEditValidationCpf($cpf, $organizationId, $businessPartnership)
    {
        if($cpf){
            $operatorRepository = InstanceProvider::getRepository()->getOperator();
            $cpfIsUsed = false;

            $parcNegocios = $this->crud->select('PARCNEGOCIO', '*', "WHERE NRINSCRICAOPARC = '{$cpf}' AND NRORG = {$organizationId} AND NRPARCNEGOCIO != '{$businessPartnership}'");
            foreach ($parcNegocios as $pn) {
                $operators = $operatorRepository->findBy(array('NRPARCNEGOCIO' => $pn['NRPARCNEGOCIO']), $organizationId);
                if(!empty($operators)) {
                    $cpfIsUsed = true;
                    break;
                }
            }

            if($cpfIsUsed){
                throw new CustomException('ERR_EXISTS_CPF');
            }
            return $parcNegocios;
        }
        return [];
    }

    public function resetPassword($password, $confirmPassword, $email, $cdOperador, $labels)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();
        try {
            $this->passwordValidate($password, $confirmPassword);
            $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
            $this->validatePasswordControlParameters($organizationId, $confirmPassword, $cdOperador);
            $operator = $this->crud->findOne('OPERADOR', '*', "WHERE CDOPERADOR = '$cdOperador' AND NRORG = '$organizationId'");
            $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
            $userId = $operator['CDOPERADOR'];
            $passWeb    = md5($password);
            $passDelphi = substr(LibUtilities::getDelphiPasswordEncrypted($password), 0, 20);
            $oldPassword = InstanceProvider::getRepository()->getOperator()->findValAntigoOperators($cdOperador);
            $arrayValores = ['operator'];
            $arrayNomes = ['Tela'];
            if ($operatorAntigo[0]['CDSENHAOPERWEB'] !== $passWeb) {
                array_push($arrayValores, '*************');
                array_push($arrayNomes, $labels['NEW_PASSWORD']);
            }
            if ($operatorAntigo[0]['CDSENHOPER'] !== $passDelphi) {
                array_push($arrayValores, '*************');
                array_push($arrayNomes, 'CDSENHOPER');
            }
            if ($operatorAntigo[0]['IDSTATUSENHA'] !== 'A') {
                array_push($arrayValores, 'A');
                array_push($arrayNomes, 'IDSTATUSENHA');
            }
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($cdOperador, $descricao, '2', 'OPERADOR', '/resetPassword/save');
            InstanceProvider::getRepository()->getOperator()->resetPassword($passWeb, $passDelphi, $cdOperador, $organizationId);
            InstanceProvider::getRepository()->getLibUserData()->insertControlSenha($userId, $organizationId, $passDelphi, $passWeb, 'R');
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    private function passwordValidate($newPassword, $confirmPassword, $password = null)
    {
        if($newPassword != $confirmPassword){
            throw new CustomException('ERR_PASSWORD_NOT_MATCH');
        }

        if($password && $password === $newPassword){
            throw new CustomException('ERR_NEW_PASSWORD_EQUAL_OLD');
        }

    }

    /**
     * If exists parameter $userId, this function called by webservice. If not, called by My Profile window.
     *
     * @param $userId
     * @param $organizationId
     * @return array
     * @throws \Exception
     */
    public function findUserData ($userId, $organizationId)
    {
        try{
            $libEnvironment = InstanceProvider::getLibEnvironment();

            $userData = array(
                'NAME'  => $userId ? null : $libEnvironment->getCurrentUserName(),
                'EMAIL' => $userId ? null : $libEnvironment->getCurrentEmail()
            );

            $organizationId = $organizationId ? $organizationId : $libEnvironment->getCurrentOrganizationId();
            $userId         = $userId ? $userId : $libEnvironment->getCurrentUserId();

            try {
                $operatorImage = $this->crud->findOne('OPERADOR', 'DSIMAGEMOPER', "WHERE CDOPERADOR = '$userId' AND NRORG = '$organizationId'", $organizationId);
                if($operatorImage['DSIMAGEMOPER']){
                    $userData['IMAGE'] = $this->fileServerFilesUrl . $operatorImage['DSIMAGEMOPER'];
                } else {
                    $userData['IMAGE'] = null;
                }
            } catch(\Exception $e) {
                $userData['IMAGE'] = null;
            }

            $userData['HAS_IMAGE'] = $userData['IMAGE'];
            return array($userData);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updatePassword($password, $newPassword, $confirmPassword)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();
        try {
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            $userId = $libEnvironment->getCurrentUserId();
            $this->passwordValidate($newPassword, $confirmPassword, $password);
            $this->validatePasswordControlParameters($organizationId, $newPassword, $userId);
            $operator = $this->crud->findOne('OPERADOR', '*', "WHERE CDOPERADOR = '$userId' AND NRORG = '$organizationId'");
            $password = md5($password);
            if($operator['CDSENHAOPERWEB'] != $password) {
                throw new CustomException('ERR_INVALID_CURRENT_PASSWORD');
            }
            $newPassWeb    = md5($newPassword);
            $newPassDelphi = substr(LibUtilities::getDelphiPasswordEncrypted($newPassword), 0, 20);
            InstanceProvider::getRepository()->getOperator()->updatePassword($newPassWeb, $newPassDelphi, $userId, $organizationId);
            InstanceProvider::getRepository()->getLibUserData()->insertControlSenha($userId, $organizationId, $newPassDelphi, $newPassWeb, 'S');
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    private function validatePasswordControlParameters($organizationId, $password, $userId)
    {
        $paramSenha = InstanceProvider::getRepository()->getParamSenha()->getParamSenhaByNrOrg($organizationId);
        if(!empty($paramSenha)) {
            /** Validate Password Mask */
            $mask = $paramSenha[0]['CDPICTSENHA'];
            if ($paramSenha[0]['USE_CONTROLSENHA'] == self::USES_PASSWORD_CONTROL_POSITIVE && !empty($mask)) {
                $length    = $this->validatePasswordLength($mask, $password);
                $upperCase = $this->validatePasswordMask($mask, $password, '[M', '/[A-Z]/', self::UPPERCASE_TYPE);
                $numeric   = $this->validatePasswordMask($mask, $password, '[N', '/[0-9]/', self::NUMERIC_TYPE);
                $special   = $this->validatePasswordMask($mask, $password, '[E', '/[!"#$%&()*+,-.:;<=>?@^{|}~]/', self::SPECIAL_TYPE);
                $this->validatePasswordMaskException($length, $upperCase, $numeric, $special);
            }
            /** Validate Changes to Reuse Same Password */
            $changesToReuse = $paramSenha[0]['NRALTSENHREP'];
            if(!empty($changesToReuse)) {
                $latestPassUp = InstanceProvider::getRepository()->getControlSenha()->getLatestPasswordUpdates($userId, $changesToReuse);
                foreach($latestPassUp as $latestUp) {
                    if(LibUtilities::encrypt($password) == $latestUp['CDSENHALTWEB']) {
                        $exceptionMessage = 'PASSWORD_NOT_REUSED';
                        $exceptionParams  = array('changes' => $changesToReuse);
                        throw new CustomException($exceptionMessage, 500, $exceptionParams);
                    }
                }
            }
        }
    }

    private function validatePasswordLength($mask, $password)
    {
        $validation = array (
            'EXISTS_MASK'  => false,
            'INVALID_MASK' => false
        );
        $lengthCaracters = explode('[', $mask)[0];
        if(!empty($lengthCaracters)) {
            $validation['EXISTS_MASK']  = true;
            $validation['MESSAGE']      = 'PASSWORD_VALIDATION_LENGTH';
            $validation['PARAMS']       = array('count' => strlen($lengthCaracters));
            $validation['INVALID_MASK'] = strlen($password) < strlen($lengthCaracters);

        }
        return $validation;
    }

    private function validatePasswordMask($mask, $password, $maskStr, $regexValidation, $type)
    {
        $validation = array (
            'EXISTS_MASK'  => false,
            'INVALID_MASK' => false
        );
        $maskCount = $this->getMaskCaractersLength($mask, $maskStr);
        if($maskCount) {
            $passwordCount = preg_match_all($regexValidation, $password);
            $validation['EXISTS_MASK']  = true;
            $validation['MESSAGE']      = $this->getPasswordMaskMessage($type);
            $validation['PARAMS']       = array('count' => $maskCount);
            $validation['INVALID_MASK'] = $passwordCount < $maskCount;
        }
        return $validation;
    }

    /**
     * @param $type
     * @return string
     */
    private function getPasswordMaskMessage($type) {
        switch($type) {
            case self::UPPERCASE_TYPE:
                return 'PASSWORD_VALIDATION_UPPERCASE';
                break;
            case self::NUMERIC_TYPE:
                return 'PASSWORD_VALIDATION_NUMERIC';
                break;
            case self::SPECIAL_TYPE:
                return 'PASSWORD_VALIDATION_SPECIAL';
                break;
        }
    }

    private function validatePasswordMaskException($length, $upperCase, $numeric, $special)
    {
        if($length['INVALID_MASK'] || $upperCase['INVALID_MASK'] || $numeric['INVALID_MASK'] || $special['INVALID_MASK']) {
            $params[] = $length['EXISTS_MASK']    ? $length    : null;
            $params[] = $upperCase['EXISTS_MASK'] ? $upperCase : null;
            $params[] = $numeric['EXISTS_MASK']   ? $numeric   : null;
            $params[] = $special['EXISTS_MASK']   ? $special   : null;
            throw new CustomException('INVALID_PASSWORD_MASK', 500, $params);
        }
    }

    private function getMaskCaractersLength($mask, $searchStr)
    {
        $length  = false;
        $initPos = strpos($mask, $searchStr);
        if($initPos !== false) {
            $initPos += 2;
            $endPos = strpos($mask, ']', $initPos);
            $length = substr($mask, $initPos, $endPos - $initPos);
        }
        return $length;
    }

    public function uploadProfilePicture($imagePath)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        try{
            $connection->beginTransaction();
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $userId = $libEnvironment->getCurrentUserId();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            $operatorImage = $this->crud->findOne('OPERADOR', 'DSIMAGEMOPER', "WHERE CDOPERADOR = '$userId' AND NRORG = '$organizationId'");
            InstanceProvider::getRepository()->getOperator()->uploadProfilePicture($imagePath, $userId, $organizationId);
            if($operatorImage['DSIMAGEMOPER']){
                $this->deleteProfilePictureInFileServe($operatorImage['DSIMAGEMOPER']);
            }
            $connection->commit();
            return [array('imagePath' => $imagePath)];
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteProfilePicture()
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        try{
            $connection->beginTransaction();
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $userId = $libEnvironment->getCurrentUserId();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            $operatorImage = $this->crud->findOne('OPERADOR', 'DSIMAGEMOPER', "WHERE CDOPERADOR = '$userId' AND NRORG = '$organizationId'");
            InstanceProvider::getRepository()->getOperator()->uploadProfilePicture(null, $userId, $organizationId);
            $this->deleteProfilePictureInFileServe($operatorImage['DSIMAGEMOPER']);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteProfilePictureInFileServe($imagePath)
    {
        $this->zhFileUploader->deleteFile($imagePath);
    }

    /**
     * @throws \Exception
     */
    public function findOrgOper($nrorgoper, &$isPrimaryOrg = false, $showFullOperator = null, $hideUndefinedOrg = false)
    {
        $operatorRepository = InstanceProvider::getRepository()->getOperator();
        $libEnvironment     = InstanceProvider::getLibEnvironment();
        $organizationId     = $libEnvironment->getCurrentOrganizationId();
        $operatorCode       = $libEnvironment->getCurrentUserId();
        $idUtlPerfilfilial  = $libEnvironment->getIdUtlPerfilFilial();
        $oAccTekMgr         = InstanceProvider::getRepository()->getOAccTekMgr()->findOneBy(array('NRORG' => $organizationId), $organizationId);
        $oAccTek            = @json_decode(LibUtilities::zeedhiDecrypt($oAccTekMgr['OACCTEK_PHP7'], LibUtilities::LOGIN_CRYPT_STRING), true);
        $oAccCustomOrgId    = $this->getOrganizationIdFromOAccCustom();

        $data = [];
        if (!empty($oAccTek['relatedOrgs'])) {
            $operatorOrgOper = $operatorRepository->findOperatorOrgOper($operatorCode, $organizationId);
            $hasPrimaryOrg   = false;
            $auxData = [];
            if(empty($operatorOrgOper)) {
                $operatorOrgOper = array ( array (
                    'NRORGOPER' => !empty($oAccCustomOrgId) ? $oAccCustomOrgId : $organizationId
                ) );
            }
            foreach($operatorOrgOper as $org) {
                if($hasPrimaryOrg) {
                    break;
                }
                foreach($oAccTek['relatedOrgs'] as $relatedOrg) {
                    if($org['NRORGOPER'] == $relatedOrg['NRORG']) {
                        if($relatedOrg['NRORGPRINCIPAL'] === 'P') {
                            $hasPrimaryOrg = true;
                            break;
                        }
                        $auxData[] = $relatedOrg;
                    }
                }
            }
            $data = $hasPrimaryOrg ? $oAccTek['relatedOrgs'] : $auxData;

            if(($idUtlPerfilfilial && $idUtlPerfilfilial != 'N') && $showFullOperator) {
                $filterFilioper = "AND CDOPERADOR = '". $operatorCode . "'";
                $relatedBranches = $operatorRepository->findFilioper($filterFilioper, '');

                $organizacoes = self::returnRelatedOrgs($relatedBranches, $oAccTek['relatedOrgs'], $isPrimaryOrg);
                $data = $organizacoes;
            }
        }

        if (empty($data)) {
            $filter = AclUtilities::getFilterTypeIn([$organizationId], 'NRORG');
            $data = InstanceProvider::getRepository()->getOrganization()->findOrganizations($filter);
        }

        if (count($data) > 1 && !$hideUndefinedOrg) {
            array_push($data, array(
                "NRORG" => "XXXX",
                "NMORGANIZACAO" => "Operadores com organização indefinida",
                "NMORGANIZACAOOPER" => "Operadores com organização indefinida",
                "NMRAZSOCIALORG" => "Operadores com organização indefinida",
                "NRINSJUREMPR" => null,
                "NRORGPRINCIPAL" => "S"
            ));
        }

        return $data;
    }

    /**
     * @return array
     * @throws \Exception\CustomException
     */
    private function getOrganizationIdFromOAccCustom() {
        $oAccCustom = array();
        try {
            $fileDir = '../../../login/backend/config/oAccCustom.tek';
            if (file_exists($fileDir)) {
                $fileContents  = file_get_contents($fileDir);
                $fileDecrypted = LibUtilities::zeedhiDecrypt($fileContents, LibUtilities::LOGIN_CRYPT_STRING);
                $fileDecoded   = @json_decode($fileDecrypted, true);
                if(isset($fileDecoded['organizationId']) && is_numeric($fileDecoded['organizationId'])) {
                    $oAccCustom = $fileDecoded;
                }
            }
        } catch(\Exception $e) {}

        return !empty($oAccCustom) ? $oAccCustom['organizationId'] : null;
    }

    public function returnRelatedOrgs($relatedBranches, $orgs, &$isPrimaryOrg) {
        $cnpjEqualArray = [];
        foreach($relatedBranches as $branch){
            $isCnpEqual = array_filter($orgs, function($o) use ($branch, &$isPrimaryOrg){
                if($o['NRINSJUREMPR'] == $branch['NRINSJURFILI']) {

                    if($o['NRORGPRINCIPAL'] == 'P'){
                         $isPrimaryOrg = true;
                    }
                    return $o;
                } else {
                    return false;
                }
            });
            if($isCnpEqual){
                $cnpjEqualArray[] = array_values($isCnpEqual)[0];
            }
        }

        return $isPrimaryOrg ? $orgs : $cnpjEqualArray;
    }


    public function findFilioper($conditions, $orderBy, $page, $pageSize)
    {
        $orderByString = '';
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $generalParam = self::findIdUtlPerfilFilialParamGeral();
        $filters = AclUtilities::createQueryFiltered($conditions);
        if($generalParam[0]['IDUTLPERFILFILIAL'] != 'N'){
            $operator = AclUtilities::getValueInFilterDataSourceByName($conditions, 'CDOPERADOR');
            $nrAccessProfile = self::findNrAccessprofileByOperator($operator);

            $utlBlock = self::findAccessProfileParams($nrAccessProfile);

            if($utlBlock == 'S'){
                $filters .= " AND IDSTATUSBLOQ = 'N' ";
            }
        }

        foreach($orderBy as $idx => $order) {
            if($idx === 'DTVALIDADE'){
                $orderByString .= ' ORDER BY TO_DATE(' . $idx . ') ' . $order . ' NULLS LAST';
            } else {
                $orderByString .= ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        $data = (($generalParam[0]['IDUTLPERFILFILIAL'] != 'N')
                ? InstanceProvider::getRepository()->getOperator()->findFilioperWithStatusBlock($filters, $orderByString, $limit, $offset)
                : InstanceProvider::getRepository()->getOperator()->findFilioper($filters, $orderByString, $limit, $offset));

        foreach ($data as &$p){
            if(!$p['DTVALIDADE']){
                $p['DTVALIDADE'] = '';
            }
        }
        return $data;
    }

    public function findUnidade($utlPerfilFilial, $operator, $operatorCont, $page, $pageSize, $conditions)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $nrAccessProfile = InstanceProvider::getEnvironment()->getNrPerfil();
        $filters = AclUtilities::createQueryFiltered($conditions);
        $utlBlock = self::findAccessProfileParams($nrAccessProfile);
        $utlBlock = empty($utlBlock) ? 'N' : $utlBlock[0]['IDBLOCKFILIAL'];
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return InstanceProvider::getRepository()->getOperator()->findUnidade($utlPerfilFilial, $operator, $operatorCont, $organizationId, $utlBlock, $limit, $offset, $filters);
    }

    public function getAllBranches($page, $pageSize, $conditions, $filter = ' '){
        $filtersConditions = AclUtilities::createQueryFiltered($conditions);
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return InstanceProvider::getRepository()->getOperator()->getAllBranches($filter, $filtersConditions, $limit, $offset);
    }

    public function deleteFilioper($cdOperador, $cdFilial, $nrOrg, $labels)
    {
        InstanceProvider::getRepository()->getOperator()->deleteFilioper($cdOperador, $cdFilial, $nrOrg);
        $pkFiliOper = $cdOperador.'_'.$cdFilial;
        $valores = [$labels['label'], $cdOperador, $cdFilial];
        $nomes = ["Tela", 'CDOPERADOR', $labels['CDFILIAL']];
        $descricao = AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($pkFiliOper, $descricao, '3', 'FILIOPER', '/findFilioper/remove');
        $operatorData = InstanceProvider::getRepository()->getOperator()->getOperatorData($cdOperador, $nrOrg);

        if($operatorData['CDFILIALSELECIONADA'] == $cdFilial){
            $arrayValores = ['operator'];
            $arrayNomes = ['Tela'];
            if ($operatorData['CDPERFILSELECIONADO'] !== null) {
                array_push($arrayValores, null);
                array_push($arrayNomes, 'CDPERFILSELECIONADO');
            }
            if ($operatorData['CDFILIALSELECIONADA'] !== null) {
                array_push($arrayValores, null);
                array_push($arrayNomes, 'CDFILIALSELECIONADA');
            }
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($cdOperador, $descricao, '2', 'OPERADOR', '/findFilioper/remove');
            InstanceProvider::getRepository()->getOperator()->deleteRelatedBranchAndProfile($cdOperador, $nrOrg);
        }
    }

    public function findParamGeral()
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findParamGeral($organizationId);
    }

    public function findParamGeralValidationCpf()
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findParamGeralValidationCpf($organizationId);
    }

    public function validateNovoCodigo()
    {
        $operatorRepository = InstanceProvider::getRepository()->getOperator();
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();

        try {
            $operatorRepository->updateActiveIdativoByIdoperativo($organizationId);
            $operatorRepository->updateInactiveIdativoByIdoperativo($organizationId);

            $operator = $operatorRepository->validarNovoCodigo($organizationId, "OPERADOR", "CDOPERADOR")[0];
            $this->saveNovoCodigo($operator["ID"], 'OPERADOR', $operator["VALIDACAO"]);

            $perfilacesso = $operatorRepository->validarNovoCodigo($organizationId, "PERFILACESSO", "NRPERFILACESSO")[0];
            $this->saveNovoCodigo($perfilacesso["ID"], 'PERFILACESSO', $perfilacesso["VALIDACAO"]);

            $perfilacessoparc = $operatorRepository->validarNovoCodigo($organizationId, "PERFACESSPARC", "NRPERFACESSPARC")[0];
            $this->saveNovoCodigo($perfilacessoparc["ID"], 'PERFACESSPARC', $perfilacessoparc["VALIDACAO"]);

            $parceria = $operatorRepository->validarNovoCodigo($organizationId, "PARCERIA", "NRPARCERIA")[0];
            $this->saveNovoCodigo($parceria["ID"], 'PARCERIA', $parceria["VALIDACAO"]);

            $organizacaoOper = $operatorRepository->validarNovoCodigo($organizationId, "ORGANIZACAOOPER", "NRORGANIZACAOOPER")[0];
            $this->saveNovoCodigo($organizacaoOper["ID"], 'ORGANIZACAOOPER', $organizacaoOper["VALIDACAO"]);
            
            $parcNegocio = $operatorRepository->validarNovoCodigo($organizationId, "PARCNEGOCIO", "NRPARCNEGOCIO")[0];
            $this->saveNovoCodigo($parcNegocio["ID"], 'PARCNEGOCIO', $parcNegocio["VALIDACAO"]);
            
            $orgRelacOper = $operatorRepository->validarNovoCodigo($organizationId, "ORGRELACOPER", "ID")[0];
            $this->saveNovoCodigo($orgRelacOper["ID"], 'ORGRELACOPER', $orgRelacOper["VALIDACAO"]);

            try {
                $relacParcNeg = $operatorRepository->validarNovoCodigo($organizationId, "RELACPARCNEG", "NRRELACPARCNEG")[0];
                $this->saveNovoCodigo($relacParcNeg["ID"], 'RELACPARCNEG', $relacParcNeg["VALIDACAO"]);
            } catch (\Exception $e) {}
            
            $oAccCustomOrgId = $this->getOrganizationIdFromOAccCustom();
            $nrorg = empty($oAccCustomOrgId) ? $organizationId : $oAccCustomOrgId;
            $operatorRepository->fixNrorgOper($organizationId, $nrorg);

            try {
                $relacOperInsert = $operatorRepository->findNrorgOperNotInserted($organizationId);
                foreach($relacOperInsert as $relac) {
                    $id = InstanceProvider::getNewCode()->getCodeByTableName('ORGRELACOPER', $nrorg, 1, 10, true);
                    $operatorRepository->insertOrgRelacOper($id, $relac['NRORGOPER'], $relac['CDOPERADOR'], $organizationId);
                }
            } catch (\Exception $e) {}

            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveNovoCodigo($newCode, $table, $validation) {
        $newCode = strval(str_pad($newCode, 20, "0", STR_PAD_LEFT));
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        InstanceProvider::getRepository()->getOperator()->alterNewCode($table, $newCode, $organizationId, $validation);
    }

    public function findIdUtlPerfilFilialParamGeral()
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findIdUtlPerfilFilialParamGeral($organizationId);
    }

    public function findNrAccessprofileByOperator($operator)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findNrAccessprofileByOperator($organizationId, $operator);
    }

    public function findAccessProfileParams($nrAccessProfile)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->findAccessProfileParams($organizationId, $nrAccessProfile);
    }

    public function getProfileCanBeEdit($operator, $operatorCont, $nrorg)
    {
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        return InstanceProvider::getRepository()->getOperator()->getProfileCanBeEdit($operator, $operatorCont, $nrorg);
    }

    public function NewGrupoper( $gropor,$operator,$nrperfilacesso)
    {
        $operatorRepository = InstanceProvider::getRepository()->getOperator();

        if(!empty($gropor)){
            $operatorRepository->DeleteGrupoper($operator);
            $operatorRepository->InsertGrupoper($gropor,$operator,$nrperfilacesso);

        }

    }
    
    public function SaveGrupoper( $gropor,$operator,$nrperfilacesso, $labels, $dsgrupoper)
    {
        foreach ($gropor as $cdgropor) {
            $valores = [$labels['label'], $cdgropor, $operator['CDOPERADOR'], $dsgrupoper];
            $nomes = ["Tela", $labels['CDGRUPOPER'], 'CDOPERADOR', $labels['DSGRUPOPER']];
            $descricao = AclUtilities::gerarDescricao($valores, $nomes);
            InstanceProvider::getLibLog()->logRegister($cdgropor.'_'.$operator['CDOPERADOR'], $descricao, '1', 'OPERGRUPOP', '/grupoper/save');
        }
        $operatorRepository = InstanceProvider::getRepository()->getOperator();

        $operatorRepository->InsertGrupoper($gropor,$operator,$nrperfilacesso);
    }
    
    public function validOperador($CDVENDEDOR, $CDOPERADOR)
    {
        return InstanceProvider::getRepository()->getOperator()->validOperador($CDVENDEDOR, $CDOPERADOR);
    }
    
    public function updateVendedor($CDVENDEDOR, $CDOPERADOR)
    {   
        try{
            if (!empty($CDVENDEDOR)){
                InstanceProvider::getRepository()->getOperator()->deleteVendedor($CDOPERADOR);
                InstanceProvider::getRepository()->getOperator()->updateVendedor($CDVENDEDOR, $CDOPERADOR);
            } else {
                InstanceProvider::getRepository()->getOperator()->deleteVendedor($CDOPERADOR);
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

}
